@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,9 @@ _imap4_attrlist[] =
   { "\\Flagged", MU_ATTRIBUTE_FLAGGED },
   { "\\Deleted", MU_ATTRIBUTE_DELETED },
   { "\\Draft", MU_ATTRIBUTE_DRAFT },
-  { "\\Seen", MU_ATTRIBUTE_SEEN|MU_ATTRIBUTE_READ },
+  /* According to RFC 3501, 2.3.2, \Seen means "Message has been read".
+     This corresponds to the mailutils attribute MU_ATTRIBUTE_READ. */
+  { "\\Seen", MU_ATTRIBUTE_READ }, 
 };
 
 static int _imap4_nattr = MU_ARRAY_SIZE (_imap4_attrlist);
