@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008, 2010-2012, 2014-2015 Free Software Foundation,
-   Inc.
+   Copyright (C) 2008-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -172,6 +171,11 @@ hdr_itrctl (void *owner, enum mu_itrctl_req req, void *arg)
       else
 	itr->backwards = !!*(int*)arg;
       break;
+
+    case mu_itrctl_count:
+      if (!arg)
+	return EINVAL;
+      return mu_header_get_field_count (itr->header, arg);
       
     default:
       return ENOSYS;
